import time
import logging

from django.conf import settings

from gargantext.core.task import shared_task, get_task_logger
from gargantext.models import UserNode


logger = get_task_logger(__name__)


@shared_task(bind=True)
def dummy(self, duration=15):
    logger.info('Start %r task (DEBUG=%r): wait %s seconds...' % (
        self.name, settings.DEBUG, duration))

    logger.info('Dummy: %r', self.request)
    me = self.db.query(UserNode).filter_by(user_id=self.user.id).one_or_none()
    logger.info('User: %r', me)
    time.sleep(duration)

    logger.info('End task %r.' % self.name)

    return str(me)


@shared_task(bind=True)
def fail(self):
    logger.info("Run failing task")
    time.sleep(2)
    raise Exception("Ay Caramba! Failed again!")


@shared_task(bind=True, autoretry_for=(Exception,), default_retry_delay=10,
    retry_kwargs=dict(max_retries=3))
def fail_random(self):
    # Failing randomly
    from random import random
    p = random()
    logger.info("Run randomly failing task (p=%s)" % p)
    if p < 2/3:
        raise Exception("Fail fail fail")
    #try:
    #    if p < 2/3:
    #        raise Exception("Fail fail fail")
    #except Exception as e:
    #    self.retry()


@shared_task(bind=True)
def simulate_work(self):
    logger.info("Start %r", self)
    for i in range(10):
        time.sleep(1)
        if i > 0 and i % 5 == 0:
            self.update_state(state="PROGRESS", meta={'progress': 1/10*i})
            logger.info("50%% done %r", self)
    logger.info("Finished %r", self)

    return "Done!"


@shared_task
def produce():
    time.sleep(0.1)
    return "Bidule"

@shared_task
def process1(data):
    return "{%s}" % data

@shared_task
def reduce(many):
    return ':'.join(many)

@shared_task
def workflow():
    from celery import chord

    #chain = produce.s("Bidule", 10) | process.map.s()
    #r = chain.apply_async()

    #r = chord([ produce.s() for _ in range(10) ])(process1.s() | process2.s())
    maps = (produce.s() | process1.s() for _ in range(20))
    r = chord(maps)(reduce.s())

    logger.info("Worflow: %r", r)

    return r
