from celery import shared_task, Task as TaskBase
from celery.utils.log import get_task_logger

from .db import Session


__all__ = ['shared_task', 'get_task_logger', 'schedule']


logger = get_task_logger(__name__)


def user_as_dict(u):
    attrs = ['id', 'username', 'is_active', 'is_staff', 'is_superuser']
    return {name: getattr(u, name) for name in attrs}


class objectview(object):
    def __init__(self, d):
        self.__dict__ = d


class Task(TaskBase):
    _db = None

    @property
    def db(self):
        if self.user is None:
            raise ValueError("Database can't be used from a task unless it " \
                "is associated with a user.")

        if not self._db:
            self._db = Session()
            self._db.login(self.user)

        return self._db

    def after_return(self, *args, **kwargs):
        if self._db:
            self._db.remove()

    def __call__(self, *args, **kwargs):
        user_dict = self.request.get('user')
        self.user = user_dict and objectview(user_dict)

        return super().__call__(*args, **kwargs)


def schedule(task, when=None, user=None, args=None, kwargs=None):
    headers = { 'user': user_as_dict(user) } if user is not None else {}
    return task.apply_async(args=args, kwargs=kwargs, headers=headers)
